# 'xmldemo.rb'
# This is an example of how to use the XMLStreamin classes
# (XMLStreamListener and XMLSpec) in 'xmlstreamin.rb' to parse an XML stream.
# [These comments are intended to be read inline -- not via rdoc...]
#
# For demonstration, it is set to extract parts of the XML example used
# in the REXML Tutorial ('demodoc.xml' here), but any other file should
# just get its tags listed.


# xmlstreamin.rb should be in the local directory or the library path:
require "xmlstreamin"
# This saves qualifying all the references with "XMLStreamin::":
include XMLStreamin

####################################################

# It is useful to create a new class that extends the basic one slightly:
class XMLGenericSpec < XMLSpec
	def start(context,name,attrs)
		print "Element: #{name} \n"
		attrs.each {|attr|
		 print "    #{attr[0]} = #{attr[1]}\n"}
 		return context
	end
end

# ...and we can use this for a default node for unknown XML formats:
specShow = XMLGenericSpec.new
# This invokes itself for any enclosed elements:
specShow.default! specShow

####################################################

# We declare a spec to handle the "name" elements of the example format:
specName = XMLGenericSpec.new
# The name itself is element text, so we redefine that method:
def specName.text(context,data)
	print "    [#{data}]\n"	## assumes single line
end
# There shouldn't be any sub-elements! Let's fail if there are:
specName.default! $specXMLFail

# The enclosing element is an 'item', so we define a node for that
# (just a basic XMLSpec, so it does nothing but model the structure):
specItem = XMLSpec.new
# We pass 'name' sub-elements on to their handling node:
specItem.specs!({'name'=>specName})
# Everything else gets ignored (the default is '$specXMLVoid, which skips them)

# Enclosing this is the 'section' element.
# For variety we use XMLGenericSpec again here, so we see the attributes:
specSection = XMLGenericSpec.new
# The only sub-element here should be an 'item':
specSection.specs!({'item'=>specItem})

# The top-level element is 'inventory'
specInventory = XMLSpec.new
specInventory.specs!({'section'=>specSection})


####################################################

# The very top is the "Document" node which references the top 'inventory' element:
specDocument = XMLSpec.new
# If we don't see the expected top tag, we go into "show" mode:
specDocument.default! specShow
specDocument.specs!({'inventory'=>specInventory})

# Finally, to parse a document, we create an XMLStreamListener and a source object,
# and pass them to REXML.
list = XMLStreamListener.new specDocument
source = File.new ARGV[0] 
REXML::Document.parse_stream(source, list)

